@@ -1,7 +1,8 @@
+/*	$Id: configfile.c 15794 2009-03-18 09:10:28Z moritz $	*/
 /*
  *  ezstream - source client for Icecast with external en-/decoder support
  *  Copyright (C) 2003, 2004, 2005, 2006  Ed Zaleski <oddsock@oddsock.org>
- *  Copyright (C) 2007                    Moritz Grimm <mdgrimm@gmx.net>
+ *  Copyright (C) 2007, 2009              Moritz Grimm <mdgrimm@gmx.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -21,14 +22,11 @@
 # include "config.h"
 #endif
 
-#include <ctype.h>
-#include <limits.h>
-#include <stdio.h>
-#include <string.h>
+#include "ezstream.h"
 
-#include "compat.h"
 #include "configfile.h"
 #include "strfctns.h"
+#include "util.h"
 #include "xalloc.h"
 
 extern char		*__progname;
@@ -117,7 +115,7 @@ parseConfig(const char *fileName)
 	streamOnce_set = 0;
 	svrinfopublic_set = 0;
 	for (cur = cur->xmlChildrenNode; cur != NULL; cur = cur->next) {
-		if (!xmlStrcmp(cur->name, BAD_CAST "url")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"url")) {
 			if (ezConfig.URL != NULL) {
 				printf("%s[%ld]: Error: Cannot have multiple <url> elements\n",
 				       fileName, xmlGetLineNo(cur));
@@ -130,7 +128,7 @@ parseConfig(const char *fileName)
 				xmlFree(ls_xmlContentPtr);
 			}
 		}
-		if (!xmlStrcmp(cur->name, BAD_CAST "sourcepassword")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"sourcepassword")) {
 			if (ezConfig.password != NULL) {
 				printf("%s[%ld]: Error: Cannot have multiple <sourcepassword> elements\n",
 				       fileName, xmlGetLineNo(cur));
@@ -143,7 +141,7 @@ parseConfig(const char *fileName)
 				xmlFree(ls_xmlContentPtr);
 			}
 		}
-		if (!xmlStrcmp(cur->name, BAD_CAST "format")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"format")) {
 			if (ezConfig.format != NULL) {
 				printf("%s[%ld]: Error: Cannot have multiple <format> elements\n",
 				       fileName, xmlGetLineNo(cur));
@@ -160,7 +158,7 @@ parseConfig(const char *fileName)
 					*p = toupper((int)*p);
 			}
 		}
-		if (!xmlStrcmp(cur->name, BAD_CAST "filename")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"filename")) {
 			if (ezConfig.fileName != NULL) {
 				printf("%s[%ld]: Error: Cannot have multiple <filename> elements\n",
 				       fileName, xmlGetLineNo(cur));
@@ -175,11 +173,11 @@ parseConfig(const char *fileName)
 					config_error++;
 					continue;
 				}
-				ezConfig.fileName = xstrdup(ls_xmlContentPtr);
+				ezConfig.fileName = UTF8toCHAR(ls_xmlContentPtr, ICONV_REPLACE);
 				xmlFree(ls_xmlContentPtr);
 			}
 		}
-		if (!xmlStrcmp(cur->name, BAD_CAST "metadata_progname")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"metadata_progname")) {
 			if (ezConfig.metadataProgram != NULL) {
 				printf("%s[%ld]: Error: Cannot have multiple <metadata_progname> elements\n",
 				       fileName, xmlGetLineNo(cur));
@@ -194,11 +192,11 @@ parseConfig(const char *fileName)
 					config_error++;
 					continue;
 				}
-				ezConfig.metadataProgram = xstrdup(ls_xmlContentPtr);
+				ezConfig.metadataProgram = UTF8toCHAR(ls_xmlContentPtr, ICONV_REPLACE);
 				xmlFree(ls_xmlContentPtr);
 			}
 		}
-		if (!xmlStrcmp(cur->name, BAD_CAST "metadata_format")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"metadata_format")) {
 			if (ezConfig.metadataFormat != NULL) {
 				printf("%s[%ld]: Error: Cannot have multiple <metadata_format> elements\n",
 				       fileName, xmlGetLineNo(cur));
@@ -219,7 +217,7 @@ parseConfig(const char *fileName)
 				}
 			}
 		}
-		if (!xmlStrcmp(cur->name, BAD_CAST "playlist_program")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"playlist_program")) {
 			if (program_set) {
 				printf("%s[%ld]: Error: Cannot have multiple <playlist_program> elements\n",
 				       fileName, xmlGetLineNo(cur));
@@ -230,7 +228,7 @@ parseConfig(const char *fileName)
 				const char *errstr;
 
 				ls_xmlContentPtr = (char *)xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
-				ezConfig.fileNameIsProgram = (int)strtonum(ls_xmlContentPtr, 0, 1, &errstr);
+				ezConfig.fileNameIsProgram = (int)strtonum(ls_xmlContentPtr, 0LL, 1LL, &errstr);
 				if (errstr) {
 					printf("%s[%ld]: Error: <playlist_program> may only contain 1 or 0\n",
 					       fileName, xmlGetLineNo(cur));
@@ -241,7 +239,7 @@ parseConfig(const char *fileName)
 				program_set = 1;
 			}
 		}
-		if (!xmlStrcmp(cur->name, BAD_CAST "shuffle")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"shuffle")) {
 			if (shuffle_set) {
 				printf("%s[%ld]: Error: Cannot have multiple <shuffle> elements\n",
 				       fileName, xmlGetLineNo(cur));
@@ -252,7 +250,7 @@ parseConfig(const char *fileName)
 				const char *errstr;
 
 				ls_xmlContentPtr = (char *)xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
-				ezConfig.shuffle = (int)strtonum(ls_xmlContentPtr, 0, 1, &errstr);
+				ezConfig.shuffle = (int)strtonum(ls_xmlContentPtr, 0LL, 1LL, &errstr);
 				if (errstr) {
 					printf("%s[%ld]: Error: <shuffle> may only contain 1 or 0\n",
 					       fileName, xmlGetLineNo(cur));
@@ -263,7 +261,7 @@ parseConfig(const char *fileName)
 				shuffle_set = 1;
 			}
 		}
-		if (!xmlStrcmp(cur->name, BAD_CAST "stream_once")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"stream_once")) {
 			if (streamOnce_set) {
 				printf("%s[%ld]: Error: Cannot have multiple <stream_once> elements\n",
 				       fileName, xmlGetLineNo(cur));
@@ -274,7 +272,7 @@ parseConfig(const char *fileName)
 				const char *errstr;
 
 				ls_xmlContentPtr = (char *)xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
-				ezConfig.streamOnce = (int)strtonum(ls_xmlContentPtr, 0, 1, &errstr);
+				ezConfig.streamOnce = (int)strtonum(ls_xmlContentPtr, 0LL, 1LL, &errstr);
 				if (errstr) {
 					printf("%s[%ld]: Error: <stream_once> may only contain 1 or 0\n",
 					       fileName, xmlGetLineNo(cur));
@@ -285,7 +283,7 @@ parseConfig(const char *fileName)
 				streamOnce_set = 1;
 			}
 		}
-		if (!xmlStrcmp(cur->name, BAD_CAST "reconnect_tries")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"reconnect_tries")) {
 			if (reconnect_set) {
 				printf("%s[%ld]: Error: Cannot have multiple <reconnect_tries> elements\n",
 				       fileName, xmlGetLineNo(cur));
@@ -296,7 +294,7 @@ parseConfig(const char *fileName)
 				const char *errstr;
 
 				ls_xmlContentPtr = (char *)xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
-				ezConfig.reconnectAttempts = (unsigned int)strtonum(ls_xmlContentPtr, 0, UINT_MAX, &errstr);
+				ezConfig.reconnectAttempts = (unsigned int)strtonum(ls_xmlContentPtr, 0LL, (long long)UINT_MAX, &errstr);
 				if (errstr) {
 					printf("%s[%ld]: Error: In <reconnect_tries>: '%s' is %s\n",
 					       fileName, xmlGetLineNo(cur), ls_xmlContentPtr, errstr);
@@ -307,7 +305,7 @@ parseConfig(const char *fileName)
 				reconnect_set = 1;
 			}
 		}
-		if (!xmlStrcmp(cur->name, BAD_CAST "svrinfoname")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"svrinfoname")) {
 			if (ezConfig.serverName != NULL) {
 				printf("%s[%ld]: Error: Cannot have multiple <svrinfoname> elements\n",
 				       fileName, xmlGetLineNo(cur));
@@ -320,7 +318,7 @@ parseConfig(const char *fileName)
 				xmlFree(ls_xmlContentPtr);
 			}
 		}
-		if (!xmlStrcmp(cur->name, BAD_CAST "svrinfourl")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"svrinfourl")) {
 			if (ezConfig.serverURL != NULL) {
 				printf("%s[%ld]: Error: Cannot have multiple <svrinfourl> elements\n",
 				       fileName, xmlGetLineNo(cur));
@@ -333,7 +331,7 @@ parseConfig(const char *fileName)
 				xmlFree(ls_xmlContentPtr);
 			}
 		}
-		if (!xmlStrcmp(cur->name, BAD_CAST "svrinfogenre")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"svrinfogenre")) {
 			if (ezConfig.serverGenre != NULL) {
 				printf("%s[%ld]: Error: Cannot have multiple <svrinfogenre> elements\n",
 				       fileName, xmlGetLineNo(cur));
@@ -346,7 +344,7 @@ parseConfig(const char *fileName)
 				xmlFree(ls_xmlContentPtr);
 			}
 		}
-		if (!xmlStrcmp(cur->name, BAD_CAST "svrinfodescription")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"svrinfodescription")) {
 			if (ezConfig.serverDescription != NULL) {
 				printf("%s[%ld]: Error: Cannot have multiple <svrinfodescription> elements\n",
 				       fileName, xmlGetLineNo(cur));
@@ -359,7 +357,7 @@ parseConfig(const char *fileName)
 				xmlFree(ls_xmlContentPtr);
 			}
 		}
-		if (!xmlStrcmp(cur->name, BAD_CAST "svrinfobitrate")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"svrinfobitrate")) {
 			if (ezConfig.serverBitrate != NULL) {
 				printf("%s[%ld]: Error: Cannot have multiple <svrinfobitrate> elements\n",
 				       fileName, xmlGetLineNo(cur));
@@ -373,7 +371,7 @@ parseConfig(const char *fileName)
 			}
 		}
 
-		if (!xmlStrcmp(cur->name, BAD_CAST "svrinfochannels")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"svrinfochannels")) {
 			if (ezConfig.serverChannels != NULL) {
 				printf("%s[%ld]: Error: Cannot have multiple <svrinfochannels> elements\n",
 				       fileName, xmlGetLineNo(cur));
@@ -386,7 +384,7 @@ parseConfig(const char *fileName)
 				xmlFree(ls_xmlContentPtr);
 			}
 		}
-		if (!xmlStrcmp(cur->name, BAD_CAST "svrinfosamplerate")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"svrinfosamplerate")) {
 			if (ezConfig.serverSamplerate != NULL) {
 				printf("%s[%ld]: Error: Cannot have multiple <svrinfosamplerate> elements\n",
 				       fileName, xmlGetLineNo(cur));
@@ -399,7 +397,7 @@ parseConfig(const char *fileName)
 				xmlFree(ls_xmlContentPtr);
 			}
 		}
-		if (!xmlStrcmp(cur->name, BAD_CAST "svrinfoquality")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"svrinfoquality")) {
 			if (ezConfig.serverQuality != NULL) {
 				printf("%s[%ld]: Error: Cannot have multiple <svrinfoquality> elements\n",
 				       fileName, xmlGetLineNo(cur));
@@ -412,7 +410,7 @@ parseConfig(const char *fileName)
 				xmlFree(ls_xmlContentPtr);
 			}
 		}
-		if (!xmlStrcmp(cur->name, BAD_CAST "svrinfopublic")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"svrinfopublic")) {
 			if (svrinfopublic_set) {
 				printf("%s[%ld]: Error: Cannot have multiple <svrinfopublic> elements\n",
 				       fileName, xmlGetLineNo(cur));
@@ -423,7 +421,7 @@ parseConfig(const char *fileName)
 				const char *errstr;
 
 				ls_xmlContentPtr = (char *)xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
-				ezConfig.serverPublic = (int)strtonum(ls_xmlContentPtr, 0, 1, &errstr);
+				ezConfig.serverPublic = (int)strtonum(ls_xmlContentPtr, 0LL, 1LL, &errstr);
 				if (errstr) {
 					printf("%s[%ld]: Error: <svrinfopublic> may only contain 1 or 0\n",
 					       fileName, xmlGetLineNo(cur));
@@ -434,14 +432,14 @@ parseConfig(const char *fileName)
 				svrinfopublic_set = 1;
 			}
 		}
-		if (!xmlStrcmp(cur->name, BAD_CAST "reencode")) {
+		if (!xmlStrcmp(cur->name, (const xmlChar *)"reencode")) {
 			xmlNodePtr	cur2;
 			int		enable_set;
 
 			enable_set = 0;
 			for (cur2 = cur->xmlChildrenNode; cur2 != NULL;
 			     cur2 = cur2->next) {
-				if (!xmlStrcmp(cur2->name, BAD_CAST "enable")) {
+				if (!xmlStrcmp(cur2->name, (const xmlChar *)"enable")) {
 					if (enable_set) {
 						printf("%s[%ld]: Error: Cannot have multiple <enable> elements\n",
 						       fileName, xmlGetLineNo(cur));
@@ -452,7 +450,7 @@ parseConfig(const char *fileName)
 						const char *errstr;
 
 						ls_xmlContentPtr = (char *)xmlNodeListGetString(doc, cur2->xmlChildrenNode, 1);
-						ezConfig.reencode = (int)strtonum(ls_xmlContentPtr, 0, 1, &errstr);
+						ezConfig.reencode = (int)strtonum(ls_xmlContentPtr, 0LL, 1LL, &errstr);
 						if (errstr) {
 							printf("%s[%ld]: Error: <enable> may only contain 1 or 0\n",
 							       fileName, xmlGetLineNo(cur));
@@ -463,15 +461,15 @@ parseConfig(const char *fileName)
 						enable_set = 1;
 					}
 				}
-				if (!xmlStrcmp(cur2->name, BAD_CAST "encdec")) {
+				if (!xmlStrcmp(cur2->name, (const xmlChar *)"encdec")) {
 					xmlNodePtr	 cur3;
 					FORMAT_ENCDEC	*pformatEncDec;
 
-					pformatEncDec = xcalloc(1, sizeof(FORMAT_ENCDEC));
+					pformatEncDec = xcalloc(1UL, sizeof(FORMAT_ENCDEC));
 
 					for (cur3 = cur2->xmlChildrenNode;
 					     cur3 != NULL; cur3 = cur3->next) {
-						if (!xmlStrcmp(cur3->name, BAD_CAST "format")) {
+						if (!xmlStrcmp(cur3->name, (const xmlChar *)"format")) {
 							if (pformatEncDec->format != NULL) {
 								printf("%s[%ld]: Error: Cannot have multiple <format> elements\n",
 								       fileName, xmlGetLineNo(cur3));
@@ -488,7 +486,7 @@ parseConfig(const char *fileName)
 									*p = toupper((int)*p);
 							}
 						}
-						if (!xmlStrcmp(cur3->name, BAD_CAST "match")) {
+						if (!xmlStrcmp(cur3->name, (const xmlChar *)"match")) {
 							if (pformatEncDec->match != NULL) {
 								printf("%s[%ld]: Error: Cannot have multiple <match> elements\n",
 								       fileName, xmlGetLineNo(cur3));
@@ -505,7 +503,7 @@ parseConfig(const char *fileName)
 									*p = tolower((int)*p);
 							}
 						}
-						if (!xmlStrcmp(cur3->name, BAD_CAST "decode")) {
+						if (!xmlStrcmp(cur3->name, (const xmlChar *)"decode")) {
 							if (pformatEncDec->decoder != NULL) {
 								printf("%s[%ld]: Error: Cannot have multiple <decode> elements\n",
 								       fileName, xmlGetLineNo(cur3));
@@ -516,7 +514,7 @@ parseConfig(const char *fileName)
 								unsigned int	ret;
 
 								ls_xmlContentPtr = (char *)xmlNodeListGetString(doc, cur3->xmlChildrenNode, 1);
-								pformatEncDec->decoder = xstrdup(ls_xmlContentPtr);
+								pformatEncDec->decoder = UTF8toCHAR(ls_xmlContentPtr, ICONV_REPLACE);
 								xmlFree(ls_xmlContentPtr);
 								if ((ret = checkDecoderLine(pformatEncDec->decoder,
 											    fileName, xmlGetLineNo(cur3)))
@@ -526,7 +524,7 @@ parseConfig(const char *fileName)
 								}
 							}
 						}
-						if (!xmlStrcmp(cur3->name, BAD_CAST "encode")) {
+						if (!xmlStrcmp(cur3->name, (const xmlChar *)"encode")) {
 							if (pformatEncDec->encoder != NULL) {
 								printf("%s[%ld]: Error: Cannot have multiple <encode> elements\n",
 								       fileName, xmlGetLineNo(cur3));
@@ -537,7 +535,7 @@ parseConfig(const char *fileName)
 								unsigned int	ret;
 
 								ls_xmlContentPtr = (char *)xmlNodeListGetString(doc, cur3->xmlChildrenNode, 1);
-								pformatEncDec->encoder = xstrdup(ls_xmlContentPtr);
+								pformatEncDec->encoder = UTF8toCHAR(ls_xmlContentPtr, ICONV_REPLACE);
 								xmlFree(ls_xmlContentPtr);
 								if ((ret = checkEncoderLine(pformatEncDec->encoder,
 											    fileName, xmlGetLineNo(cur3)))
