@@ -1,7 +1,8 @@
+/*	$Id: ezstream.c 15795 2009-03-18 19:55:20Z moritz $	*/
 /*
  *  ezstream - source client for Icecast with external en-/decoder support
  *  Copyright (C) 2003, 2004, 2005, 2006  Ed Zaleski <oddsock@oddsock.org>
- *  Copyright (C) 2007                    Moritz Grimm <mdgrimm@gmx.net>
+ *  Copyright (C) 2007, 2009              Moritz Grimm <mdgrimm@gmx.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -21,41 +22,15 @@
 # include "config.h"
 #endif
 
-#ifdef HAVE_SYS_TYPES_H
-# include <sys/types.h>
-#endif
-#ifdef HAVE_SYS_STAT_H
-# include <sys/stat.h>
-#endif
-#ifdef HAVE_SYS_TIME_H
-# include <sys/time.h>
-#endif
-#include <ctype.h>
-#include <errno.h>
-#include <fcntl.h>
-#ifdef HAVE_LIBGEN_H
-# include <libgen.h>
-#endif
-#include <limits.h>
-#ifdef HAVE_PATHS_H
-# include <paths.h>
-#endif
+#include "ezstream.h"
+
 #ifdef HAVE_SIGNAL_H
 # include <signal.h>
 #endif
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
+
 #include <shout/shout.h>
 
-#include "compat.h"
 #include "configfile.h"
-#ifndef HAVE_GETOPT
-# include "getopt.h"
-#endif
 #include "metadata.h"
 #include "playlist.h"
 #include "strfctns.h"
@@ -111,7 +86,7 @@ typedef struct tag_ID3Tag {
 	char	genre;
 } ID3Tag;
 
-int		urlParse(const char *, char **, int *, char **);
+int		urlParse(const char *, char **, unsigned short *, char **);
 void		replaceString(const char *, char *, size_t, const char *,
 			      const char *);
 char *		buildCommandString(const char *, const char *, metadata_t *);
@@ -119,9 +94,9 @@ char *		getMetadataString(const char *, metadata_t *);
 metadata_t *	getMetadata(const char *);
 int		setMetadata(shout_t *, metadata_t *, char **);
 FILE *		openResource(shout_t *, const char *, int *, metadata_t **,
-			     int *, int *);
+			     int *, long *);
 int		reconnectServer(shout_t *, int);
-const char *	getTimeString(int);
+const char *	getTimeString(long);
 int		sendStream(shout_t *, FILE *, const char *, int, const char *,
 			   struct timeval *);
 int		streamFile(shout_t *, const char *);
@@ -163,9 +138,10 @@ sig_handler(int sig)
 #endif /* HAVE_SIGNALS */
 
 int
-urlParse(const char *url, char **hostname, int *port, char **mountname)
+urlParse(const char *url, char **hostname, unsigned short *port,
+	 char **mountname)
 {
-	char		*p1, *p2, *p3;
+	const char	*p1, *p2, *p3;
 	char		 tmpPort[6] = "";
 	size_t		 hostsiz, mountsiz;
 	const char	*errstr;
@@ -182,7 +158,7 @@ urlParse(const char *url, char **hostname, int *port, char **mountname)
 		return (0);
 	}
 
-	p1 = (char *)(url) + strlen("http://");
+	p1 = url + strlen("http://");
 	p2 = strchr(p1, ':');
 	if (p2 == NULL) {
 		printf("%s: Error: Invalid <url>: Missing port\n",
@@ -202,8 +178,8 @@ urlParse(const char *url, char **hostname, int *port, char **mountname)
 		return (0);
 	}
 
-	strlcpy(tmpPort, p2, (p3 - p2) + 1);
-	*port = (int)strtonum(tmpPort, 1, 65535, &errstr);
+	strlcpy(tmpPort, p2, (p3 - p2) + 1UL);
+	*port = (unsigned short)strtonum(tmpPort, 1LL, (long long)USHRT_MAX, &errstr);
 	if (errstr) {
 		printf("%s: Error: Invalid <url>: Port '%s' is %s\n",
 		       __progname, tmpPort, errstr);
@@ -222,8 +198,8 @@ void
 replaceString(const char *source, char *dest, size_t size,
 	      const char *from, const char *to)
 {
-	char	*p1 = (char *)source;
-	char	*p2;
+	const char	*p1 = source;
+	const char	*p2;
 
 	p2 = strstr(p1, from);
 	if (p2 != NULL) {
@@ -232,7 +208,7 @@ replaceString(const char *source, char *dest, size_t size,
 			       __progname);
 			abort();
 		}
-		strncat(dest, p1, p2 - p1);
+		strncat(dest, p1, (size_t)(p2 - p1));
 		strlcat(dest, to, size);
 		p1 = p2 + strlen(from);
 	}
@@ -251,11 +227,20 @@ buildCommandString(const char *extension, const char *fileName,
 	size_t	 newDecoderLen = 0;
 	char	*newEncoder = NULL;
 	size_t	 newEncoderLen = 0;
+	char	*localTitle = UTF8toCHAR(metadata_get_title(mdata),
+					 ICONV_REPLACE);
+	char	*localArtist = UTF8toCHAR(metadata_get_artist(mdata),
+					  ICONV_REPLACE);
+	char	*localMetaString = UTF8toCHAR(metadata_get_string(mdata),
+					      ICONV_REPLACE);
 
 	decoder = xstrdup(getFormatDecoder(extension));
 	if (strlen(decoder) == 0) {
 		printf("%s: Unknown extension '%s', cannot decode '%s'\n",
 		       __progname, extension, fileName);
+		xfree(localTitle);
+		xfree(localArtist);
+		xfree(localMetaString);
 		xfree(decoder);
 		return (NULL);
 	}
@@ -264,18 +249,18 @@ buildCommandString(const char *extension, const char *fileName,
 	replaceString(decoder, newDecoder, newDecoderLen, TRACK_PLACEHOLDER,
 		      fileName);
 	if (strstr(decoder, ARTIST_PLACEHOLDER) != NULL) {
-		size_t tmpLen = strlen(newDecoder) + strlen(metadata_get_artist(mdata)) + 1;
+		size_t tmpLen = strlen(newDecoder) + strlen(localArtist) + 1;
 		char *tmpStr = xcalloc(tmpLen, sizeof(char));
 		replaceString(newDecoder, tmpStr, tmpLen, ARTIST_PLACEHOLDER,
-			      metadata_get_artist(mdata));
+			      localArtist);
 		xfree(newDecoder);
 		newDecoder = tmpStr;
 	}
 	if (strstr(decoder, TITLE_PLACEHOLDER) != NULL) {
-		size_t tmpLen = strlen(newDecoder) + strlen(metadata_get_title(mdata)) + 1;
+		size_t tmpLen = strlen(newDecoder) + strlen(localTitle) + 1;
 		char *tmpStr = xcalloc(tmpLen, sizeof(char));
 		replaceString(newDecoder, tmpStr, tmpLen, TITLE_PLACEHOLDER,
-			      metadata_get_title(mdata));
+			      localTitle);
 		xfree(newDecoder);
 		newDecoder = tmpStr;
 	}
@@ -308,11 +293,11 @@ buildCommandString(const char *extension, const char *fileName,
 				xfree(newDecoder);
 				newDecoder = tmpStr;
 			} else {
-				size_t tmpLen = strlen(newDecoder) + strlen(metadata_get_string(mdata)) + 1;
+				size_t tmpLen = strlen(newDecoder) + strlen(localMetaString) + 1;
 				char *tmpStr = xcalloc(tmpLen, sizeof(char));
 				replaceString(newDecoder, tmpStr, tmpLen,
 					      METADATA_PLACEHOLDER,
-					      metadata_get_string(mdata));
+					      localMetaString);
 				xfree(newDecoder);
 				newDecoder = tmpStr;
 			}
@@ -329,21 +314,24 @@ buildCommandString(const char *extension, const char *fileName,
 		commandStringLen = strlen(newDecoder) + 1;
 		commandString = xcalloc(commandStringLen, sizeof(char));
 		strlcpy(commandString, newDecoder, commandStringLen);
+		xfree(localTitle);
+		xfree(localArtist);
+		xfree(localMetaString);
 		xfree(decoder);
 		xfree(encoder);
 		xfree(newDecoder);
 		return (commandString);
 	}
 
-	newEncoderLen = strlen(encoder) + strlen(metadata_get_artist(mdata)) + 1;
+	newEncoderLen = strlen(encoder) + strlen(localArtist) + 1;
 	newEncoder = xcalloc(newEncoderLen, sizeof(char));
 	replaceString(encoder, newEncoder, newEncoderLen, ARTIST_PLACEHOLDER,
-		      metadata_get_artist(mdata));
+		      localArtist);
 	if (strstr(encoder, TITLE_PLACEHOLDER) != NULL) {
-		size_t tmpLen = strlen(newEncoder) + strlen(metadata_get_title(mdata)) + 1;
+		size_t tmpLen = strlen(newEncoder) + strlen(localTitle) + 1;
 		char *tmpStr = xcalloc(tmpLen, sizeof(char));
 		replaceString(newEncoder, tmpStr, tmpLen, TITLE_PLACEHOLDER,
-			      metadata_get_title(mdata));
+			      localTitle);
 		xfree(newEncoder);
 		newEncoder = tmpStr;
 	}
@@ -366,11 +354,11 @@ buildCommandString(const char *extension, const char *fileName,
 				xfree(newEncoder);
 				newEncoder = tmpStr;
 			} else {
-				size_t tmpLen = strlen(newEncoder) + strlen(metadata_get_string(mdata)) + 1;
+				size_t tmpLen = strlen(newEncoder) + strlen(localMetaString) + 1;
 				char *tmpStr = xcalloc(tmpLen, sizeof(char));
 				replaceString(newEncoder, tmpStr, tmpLen,
 					      METADATA_PLACEHOLDER,
-					      metadata_get_string(mdata));
+					      localMetaString);
 				xfree(newEncoder);
 				newEncoder = tmpStr;
 			}
@@ -383,6 +371,9 @@ buildCommandString(const char *extension, const char *fileName,
 	snprintf(commandString, commandStringLen, "%s | %s", newDecoder,
 		 newEncoder);
 
+	xfree(localTitle);
+	xfree(localArtist);
+	xfree(localMetaString);
 	xfree(decoder);
 	xfree(encoder);
 	xfree(newDecoder);
@@ -555,7 +546,7 @@ setMetadata(shout_t *shout, metadata_t *mdata, char **mdata_copy)
 
 FILE *
 openResource(shout_t *shout, const char *fileName, int *popenFlag,
-	     metadata_t **mdata_p, int *isStdin, int *songLen)
+	     metadata_t **mdata_p, int *isStdin, long *songLen)
 {
 	FILE		*filep = NULL;
 	char		 extension[25];
@@ -730,10 +721,10 @@ reconnectServer(shout_t *shout, int closeConn)
 }
 
 const char *
-getTimeString(int seconds)
+getTimeString(long seconds)
 {
 	static char	str[20];
-	int		secs, mins, hours;
+	long		secs, mins, hours;
 
 	if (seconds < 0)
 		return (NULL);
@@ -744,7 +735,7 @@ getTimeString(int seconds)
 	mins = secs / 60;
 	secs %= 60;
 
-	snprintf(str, sizeof(str), "%uh%02um%02us", hours, mins, secs);
+	snprintf(str, sizeof(str), "%ldh%02ldm%02lds", hours, mins, secs);
 	return ((const char *)str);
 }
 
@@ -753,7 +744,7 @@ sendStream(shout_t *shout, FILE *filepstream, const char *fileName,
 	   int isStdin, const char *songLenStr, struct timeval *tv)
 {
 	unsigned char	 buff[4096];
-	size_t		 read, total, oldTotal;
+	size_t		 bytes_read, total, oldTotal;
 	int		 ret;
 	double		 kbps = -1.0;
 	struct timeval	 timeStamp, *startTime = tv;
@@ -769,7 +760,7 @@ sendStream(shout_t *shout, FILE *filepstream, const char *fileName,
 
 	total = oldTotal = 0;
 	ret = STREAM_DONE;
-	while ((read = fread(buff, 1, sizeof(buff), filepstream)) > 0) {
+	while ((bytes_read = fread(buff, 1UL, sizeof(buff), filepstream)) > 0) {
 		if (shout_get_connected(shout) != SHOUTERR_CONNECTED &&
 		    reconnectServer(shout, 0) == 0) {
 			ret = STREAM_SERVERR;
@@ -778,7 +769,7 @@ sendStream(shout_t *shout, FILE *filepstream, const char *fileName,
 
 		shout_sync(shout);
 
-		if (shout_send(shout, buff, read) != SHOUTERR_SUCCESS) {
+		if (shout_send(shout, buff, bytes_read) != SHOUTERR_SUCCESS) {
 			printf("%s: shout_send(): %s\n", __progname,
 			       shout_get_error(shout));
 			if (reconnectServer(shout, 1))
@@ -810,16 +801,16 @@ sendStream(shout_t *shout, FILE *filepstream, const char *fileName,
 			}
 		}
 
-		total += read;
+		total += bytes_read;
 		if (qFlag && vFlag) {
-			struct timeval	tv;
+			struct timeval	tval;
 			double		oldTime, newTime;
 
 			if (!isStdin && playlistMode) {
 				if (pezConfig->fileNameIsProgram) {
 					char *tmp = xstrdup(pezConfig->fileName);
 					printf("  [%s]",
-					       basename(tmp));
+					       local_basename(tmp));
 					xfree(tmp);
 				} else
 					printf("  [%4lu/%-4lu]",
@@ -829,20 +820,20 @@ sendStream(shout_t *shout, FILE *filepstream, const char *fileName,
 
 			oldTime = (double)timeStamp.tv_sec
 				+ (double)timeStamp.tv_usec / 1000000.0;
-			ez_gettimeofday((void *)&tv);
-			newTime = (double)tv.tv_sec
-				+ (double)tv.tv_usec / 1000000.0;
+			ez_gettimeofday((void *)&tval);
+			newTime = (double)tval.tv_sec
+				+ (double)tval.tv_usec / 1000000.0;
 			if (songLenStr == NULL)
 				printf("  [ %s]",
-				       getTimeString(tv.tv_sec - startTime->tv_sec));
+				       getTimeString(tval.tv_sec - startTime->tv_sec));
 			else
 				printf("  [ %s/%s]",
-				       getTimeString(tv.tv_sec - startTime->tv_sec),
+				       getTimeString(tval.tv_sec - startTime->tv_sec),
 				       songLenStr);
 			if (newTime - oldTime >= 1.0) {
 				kbps = (((double)(total - oldTotal) / (newTime - oldTime)) * 8.0) / 1000.0;
-				timeStamp.tv_sec = tv.tv_sec;
-				timeStamp.tv_usec = tv.tv_usec;
+				timeStamp.tv_sec = tval.tv_sec;
+				timeStamp.tv_usec = tval.tv_usec;
 				oldTotal = total;
 			}
 			if (kbps < 0)
@@ -876,7 +867,8 @@ streamFile(shout_t *shout, const char *fileName)
 	int		 popenFlag = 0;
 	char		*songLenStr = NULL;
 	int		 isStdin = 0;
-	int		 ret, retval = 0, songLen;
+	int		 ret, retval = 0;
+	long		 songLen;
 	metadata_t	*mdata;
 	struct timeval	 startTime;
 
@@ -1045,13 +1037,14 @@ char *
 getProgname(const char *argv0)
 {
 #ifdef HAVE___PROGNAME
+	(void)argv0;
 	return (strdup(__progname));
 #else
 	char	*p;
 
 	if (argv0 == NULL)
 		return ((char *)"ezstream");
-	p = strrchr(argv0, PATH_SEPARATOR);
+	p = strrchr(argv0, path_separators[0]);
 	if (p == NULL)
 		p = (char *)argv0;
 	else
@@ -1098,7 +1091,7 @@ main(int argc, char *argv[])
 	int		 c;
 	char		*configFile = NULL;
 	char		*host = NULL;
-	int		 port = 0;
+	unsigned short	 port = 0;
 	char		*mount = NULL;
 	shout_t 	*shout;
 	extern char	*optarg;
@@ -1123,7 +1116,7 @@ main(int argc, char *argv[])
 	qFlag = 0;
 	vFlag = 0;
 
-	while ((c = getopt(argc, argv, "c:hnqVv")) != -1) {
+	while ((c = local_getopt(argc, argv, "c:hnqVv")) != -1) {
 		switch (c) {
 		case 'c':
 			if (configFile != NULL) {
@@ -1269,7 +1262,7 @@ main(int argc, char *argv[])
 	if (shout_open(shout) == SHOUTERR_SUCCESS) {
 		int	ret;
 
-		printf("%s: Connected to http://%s:%d%s\n", __progname,
+		printf("%s: Connected to http://%s:%hu%s\n", __progname,
 		       host, port, mount);
 
 		if (pezConfig->fileNameIsProgram ||
@@ -1299,7 +1292,7 @@ main(int argc, char *argv[])
 
 		shout_close(shout);
 	} else
-		printf("%s: Connection to http://%s:%d%s failed: %s\n", __progname,
+		printf("%s: Connection to http://%s:%hu%s failed: %s\n", __progname,
 		       host, port, mount, shout_get_error(shout));
 
 	if (quit)
