@@ -1,5 +1,6 @@
+/*	$Id: playlist.c 15794 2009-03-18 09:10:28Z moritz $	*/
 /*
- * Copyright (c) 2007 Moritz Grimm <mdgrimm@gmx.net>
+ * Copyright (c) 2007, 2009 Moritz Grimm <mdgrimm@gmx.net>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -18,23 +19,16 @@
 # include "config.h"
 #endif
 
-#ifdef HAVE_SYS_TYPES_H
-# include <sys/types.h>
-#endif
-#ifdef HAVE_SYS_STAT_H
-# include <sys/stat.h>
-#endif
-#include <errno.h>
-#include <limits.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <time.h>
+#include "ezstream.h"
 
-#include "compat.h"
 #include "playlist.h"
 #include "xalloc.h"
 
+/* Usually defined in <sys/stat.h>. */
+#ifndef S_IEXEC
+# define S_IEXEC	S_IXUSR
+#endif /* !S_IEXEC */
+
 extern char	*__progname;
 
 struct playlist {
@@ -57,7 +51,7 @@ playlist_create(const char *filename)
 {
 	playlist_t	*pl;
 
-	pl = xcalloc(1, sizeof(playlist_t));
+	pl = xcalloc(1UL, sizeof(playlist_t));
 	pl->filename = xstrdup(filename);
 
 	return (pl);
@@ -77,14 +71,14 @@ playlist_add(playlist_t *pl, const char *entry)
 	num = pl->num + 1;
 
 	if (pl->size == 0) {
-		pl->list = xcalloc(2, sizeof(char *));
+		pl->list = xcalloc(2UL, sizeof(char *));
 		pl->size = 2 * sizeof(char *);
 	}
 
 	if (pl->size / sizeof(char *) <= num) {
 		size_t	i;
 
-		pl->list = xrealloc(pl->list, 2, pl->size);
+		pl->list = xrealloc(pl->list, 2UL, pl->size);
 		pl->size = 2 * pl->size;
 
 		for (i = num; i < pl->size / sizeof(char *); i++)
@@ -146,7 +140,7 @@ playlist_read(const char *filename)
 	}
 
 	line = 0;
-	while (fgets(buf, sizeof(buf), filep) != NULL) {
+	while (fgets(buf, (int)sizeof(buf), filep) != NULL) {
 		line++;
 
 		if (strlen(buf) == sizeof(buf) - 1) {
@@ -155,7 +149,7 @@ playlist_read(const char *filename)
 			printf("%s[%lu]: File or path name too long\n",
 			       filename, line);
 			/* Discard any excess chars in that line. */
-			while (fgets(skip_buf, sizeof(skip_buf), filep) != NULL) {
+			while (fgets(skip_buf, (int)sizeof(skip_buf), filep) != NULL) {
 				if (skip_buf[0] == '\n')
 					break;
 			}
@@ -355,7 +349,7 @@ playlist_get_position(playlist_t *pl)
 }
 
 int
-playlist_set_position(playlist_t *pl, unsigned long index)
+playlist_set_position(playlist_t *pl, unsigned long idx)
 {
 	if (pl == NULL) {
 		printf("%s: playlist_set_position(): Internal error: NULL argument\n",
@@ -363,10 +357,10 @@ playlist_set_position(playlist_t *pl, unsigned long index)
 		exit(1);
 	}
 
-	if (pl->program || index > pl->num - 1)
+	if (pl->program || idx > pl->num - 1)
 		return (0);
 
-	pl->index = (size_t)index;
+	pl->index = (size_t)idx;
 
 	return (1);
 }
@@ -504,7 +498,7 @@ playlist_run_program(playlist_t *pl)
 		return (NULL);
 	}
 
-	if (fgets(buf, sizeof(buf), filep) == NULL) {
+	if (fgets(buf, (int)sizeof(buf), filep) == NULL) {
 		if (ferror(filep))
 			printf("%s: Error while reading output from program '%s': %s\n",
 			       __progname, pl->filename, strerror(errno));
