@@ -1,7 +1,8 @@
+/*	$Id: util.c 15794 2009-03-18 09:10:28Z moritz $	*/
 /*
  *  ezstream - source client for Icecast with external en-/decoder support
  *  Copyright (C) 2003, 2004, 2005, 2006  Ed Zaleski <oddsock@oddsock.org>
- *  Copyright (C) 2007                    Moritz Grimm <mdgrimm@gmx.net>
+ *  Copyright (C) 2007, 2009              Moritz Grimm <mdgrimm@gmx.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -26,32 +27,20 @@
 # include "config.h"
 #endif
 
-#ifdef HAVE_SYS_TYPES_H
-# include <sys/types.h>
-#endif
-#ifdef HAVE_SYS_TIME_H
-# include <sys/time.h>
-#else
-# include <time.h>
-#endif
+#include "ezstream.h"
 
-#include <ctype.h>
-#include <errno.h>
 #ifdef HAVE_LANGINFO_H
 # include <langinfo.h>
 #endif
 #ifdef HAVE_LOCALE_H
 # include <locale.h>
 #endif
-#include <stdio.h>
-#include <string.h>
 
 #ifdef HAVE_ICONV
 # include <iconv.h>
 #endif
 #include <shout/shout.h>
 
-#include "compat.h"
 #include "util.h"
 #include "configfile.h"
 #include "xalloc.h"
@@ -100,7 +89,7 @@ strrcasecmp(const char *s, const char *sub)
 }
 
 shout_t *
-stream_setup(const char *host, const int port, const char *mount)
+stream_setup(const char *host, unsigned short port, const char *mount)
 {
 	shout_t *shout = NULL;
 
@@ -219,7 +208,7 @@ stream_setup(const char *host, const int port, const char *mount)
 		return (NULL);
 	}
 
-	if (shout_set_public(shout, pezConfig->serverPublic) != SHOUTERR_SUCCESS) {
+	if (shout_set_public(shout, (unsigned int)pezConfig->serverPublic) != SHOUTERR_SUCCESS) {
 		printf("%s: shout_set_public(): %s\n",
 		       __progname, shout_get_error(shout));
 		shout_free(shout);
@@ -237,7 +226,7 @@ CHARtoUTF8(const char *in_str, int mode)
 
 # if defined(HAVE_NL_LANGINFO) && defined(HAVE_SETLOCALE) && defined(CODESET)
 	setlocale(LC_CTYPE, "");
-	codeset = nl_langinfo(CODESET);
+	codeset = nl_langinfo((nl_item)CODESET);
 	setlocale(LC_CTYPE, "C");
 # else
 	codeset = (char *)"";
@@ -248,9 +237,6 @@ CHARtoUTF8(const char *in_str, int mode)
 	snprintf(codeset, sizeof(codeset), "CP%u", GetACP());
 #endif /* !WIN32 */
 
-	if (in_str == NULL || strlen(in_str) == 0)
-		return (NULL);
-
 	return (iconvert(in_str, codeset, "UTF-8", mode));
 }
 
@@ -262,7 +248,7 @@ UTF8toCHAR(const char *in_str, int mode)
 
 # if defined(HAVE_NL_LANGINFO) && defined(HAVE_SETLOCALE) && defined(CODESET)
 	setlocale(LC_CTYPE, "");
-	codeset = nl_langinfo(CODESET);
+	codeset = nl_langinfo((nl_item)CODESET);
 	setlocale(LC_CTYPE, "C");
 # else
 	codeset = (char *)"";
@@ -273,9 +259,6 @@ UTF8toCHAR(const char *in_str, int mode)
 	snprintf(codeset, sizeof(codeset), "CP%u", GetACP());
 #endif /* !WIN32 */
 
-	if (in_str == NULL || strlen(in_str) == 0)
-		return (NULL);
-
 	return (iconvert(in_str, "UTF-8", codeset, mode));
 }
 
@@ -293,6 +276,9 @@ iconvert(const char *in_str, const char *from, const char *to, int mode)
 	size_t			 out_pos;
 	char			*tocode;
 
+	if (NULL == in_str)
+		return (xstrdup(""));
+
 	switch (mode) {
 		size_t	siz;
 
@@ -318,7 +304,7 @@ iconvert(const char *in_str, const char *from, const char *to, int mode)
 	    (cd = iconv_open(tocode, "")) == (iconv_t)-1) {
 		xfree(tocode);
 		printf("%s: iconv_open(): %s\n", __progname, strerror(errno));
-		return (NULL);
+		return (xstrdup(in_str));
 	}
 
 	ip = input = (ICONV_CONST char *)in_str;
@@ -359,12 +345,19 @@ iconvert(const char *in_str, const char *from, const char *to, int mode)
 		printf("%s: iconv_close(): %s\n", __progname, strerror(errno));
 		xfree(output);
 		xfree(tocode);
-		return (NULL);
+		return (xstrdup(in_str));
 	}
 
 	xfree(tocode);
 	return (output);
 #else
+	(void)from;
+	(void)to;
+	(void)mode;
+
+	if (NULL == in_str)
+		return (xstrdup(""));
+
 	return (xstrdup(in_str));
 #endif /* HAVE_ICONV */
 }
